@@ -1,4 +1,6 @@
-# Copyright 2011 OpenStack Foundation
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
+# Copyright 2011 OpenStack LLC.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -13,14 +15,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import glance_store as store
-from oslo_log import log as logging
 import webob.exc
 
 from glance.common import exception
-from glance.i18n import _
-import glance.registry.client.v1.api as registry
-
+import glance.openstack.common.log as logging
+from glance import registry
+import glance.store as store
 
 LOG = logging.getLogger(__name__)
 
@@ -34,38 +34,34 @@ class BaseController(object):
         :param request: The WSGI/Webob Request object
         :param image_id: The opaque image identifier
 
-        :raises HTTPNotFound: if image does not exist
+        :raises HTTPNotFound if image does not exist
         """
         context = request.context
         try:
             return registry.get_image_metadata(context, image_id)
         except exception.NotFound:
-            LOG.debug("Image with identifier %s not found", image_id)
             msg = _("Image with identifier %s not found") % image_id
+            LOG.debug(msg)
             raise webob.exc.HTTPNotFound(
-                msg, request=request, content_type='text/plain')
+                    msg, request=request, content_type='text/plain')
         except exception.Forbidden:
-            LOG.debug("Forbidden image access")
-            raise webob.exc.HTTPForbidden(_("Forbidden image access"),
+            msg = _("Forbidden image access")
+            LOG.debug(msg)
+            raise webob.exc.HTTPForbidden(msg,
                                           request=request,
                                           content_type='text/plain')
 
-    def get_active_image_meta_or_error(self, request, image_id):
+    def get_active_image_meta_or_404(self, request, image_id):
         """
-        Same as get_image_meta_or_404 except that it will raise a 403 if the
-        image is deactivated or 404 if the image is otherwise not 'active'.
+        Same as get_image_meta_or_404 except that it will raise a 404 if the
+        image isn't 'active'.
         """
         image = self.get_image_meta_or_404(request, image_id)
-        if image['status'] == 'deactivated':
-            LOG.debug("Image %s is deactivated", image_id)
-            msg = _("Image %s is deactivated") % image_id
-            raise webob.exc.HTTPForbidden(
-                msg, request=request, content_type='text/plain')
         if image['status'] != 'active':
-            LOG.debug("Image %s is not active", image_id)
             msg = _("Image %s is not active") % image_id
+            LOG.debug(msg)
             raise webob.exc.HTTPNotFound(
-                msg, request=request, content_type='text/plain')
+                    msg, request=request, content_type='text/plain')
         return image
 
     def update_store_acls(self, req, image_id, location_uri, public=False):
@@ -80,17 +76,11 @@ class BaseController(object):
                             write_tenants.append(member['member_id'])
                         else:
                             read_tenants.append(member['member_id'])
-                store.set_acls(location_uri, public=public,
+                store.set_acls(req.context, location_uri, public=public,
                                read_tenants=read_tenants,
-                               write_tenants=write_tenants,
-                               context=req.context)
-            except store.UnknownScheme:
+                               write_tenants=write_tenants)
+            except exception.UnknownScheme:
                 msg = _("Store for image_id not found: %s") % image_id
                 raise webob.exc.HTTPBadRequest(explanation=msg,
                                                request=req,
                                                content_type='text/plain')
-            except store.NotFound:
-                msg = _("Data for image_id not found: %s") % image_id
-                raise webob.exc.HTTPNotFound(explanation=msg,
-                                             request=req,
-                                             content_type='text/plain')
