@@ -1,6 +1,6 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-# Copyright 2010 OpenStack LLC.
+# Copyright 2010-2012 OpenStack LLC.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -19,55 +19,45 @@
 /images endpoint for Glance v1 API
 """
 
-import sys
-import traceback
+import copy
 
 import eventlet
+from oslo.config import cfg
 from webob.exc import (HTTPError,
                        HTTPNotFound,
                        HTTPConflict,
                        HTTPBadRequest,
                        HTTPForbidden,
                        HTTPRequestEntityTooLarge,
-                       HTTPServiceUnavailable,
-                      )
+                       HTTPInternalServerError,
+                       HTTPServiceUnavailable)
+from webob import Response
 
 from glance.api import common
 from glance.api import policy
 import glance.api.v1
-from glance import context
 from glance.api.v1 import controller
 from glance.api.v1 import filters
 from glance.common import exception
 from glance.common import utils
 from glance.common import wsgi
 from glance import notifier
-from glance.openstack.common import cfg
 import glance.openstack.common.log as logging
 from glance import registry
-from glance.store import (create_stores,
-                          get_from_backend,
+from glance.store import (get_from_backend,
                           get_size_from_backend,
                           safe_delete_from_backend,
                           schedule_delayed_delete_from_backend,
                           get_store_from_location,
                           get_store_from_scheme)
 
-
+CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 SUPPORTED_PARAMS = glance.api.v1.SUPPORTED_PARAMS
 SUPPORTED_FILTERS = glance.api.v1.SUPPORTED_FILTERS
 CONTAINER_FORMATS = ['ami', 'ari', 'aki', 'bare', 'ovf']
 DISK_FORMATS = ['ami', 'ari', 'aki', 'vhd', 'vmdk', 'raw', 'qcow2', 'vdi',
-               'iso']
-
-
-# Defined at module level due to _is_opt_registered
-# identity check (not equality).
-default_store_opt = cfg.StrOpt('default_store', default='file')
-
-CONF = cfg.CONF
-CONF.register_opt(default_store_opt)
+                'iso']
 
 
 def validate_image_meta(req, values):
@@ -77,12 +67,12 @@ def validate_image_meta(req, values):
     container_format = values.get('container_format')
 
     if 'disk_format' in values:
-        if not disk_format in DISK_FORMATS:
+        if disk_format not in DISK_FORMATS:
             msg = "Invalid disk format '%s' for image." % disk_format
             raise HTTPBadRequest(explanation=msg, request=req)
 
     if 'container_format' in values:
-        if not container_format in CONTAINER_FORMATS:
+        if container_format not in CONTAINER_FORMATS:
             msg = "Invalid container format '%s' for image." % container_format
             raise HTTPBadRequest(explanation=msg, request=req)
 
@@ -98,15 +88,28 @@ def validate_image_meta(req, values):
         elif container_format is None:
             values['container_format'] = disk_format
         elif container_format != disk_format:
-            msg = ("Invalid mix of disk and container formats. "
-                   "When setting a disk or container format to "
-                   "one of 'aki', 'ari', or 'ami', the container "
-                   "and disk formats must match.")
+            msg = (_("Invalid mix of disk and container formats. "
+                     "When setting a disk or container format to "
+                     "one of 'aki', 'ari', or 'ami', the container "
+                     "and disk formats must match."))
             raise HTTPBadRequest(explanation=msg, request=req)
 
     return values
 
 
+def redact_loc(image_meta):
+    """
+    Create a shallow copy of image meta with 'location' removed
+    for security (as it can contain credentials).
+    """
+    if 'location' in image_meta:
+        tmp_image_meta = copy.copy(image_meta)
+        del tmp_image_meta['location']
+        return tmp_image_meta
+
+    return image_meta
+
+
 class Controller(controller.BaseController):
     """
     WSGI controller for images resource in Glance v1 API
@@ -127,8 +130,6 @@ class Controller(controller.BaseController):
     """
 
     def __init__(self):
-        create_stores()
-        self.verify_scheme_or_exit(CONF.default_store)
         self.notifier = notifier.Notifier()
         registry.configure_registry_client()
         self.policy = policy.Enforcer()
@@ -224,6 +225,10 @@ class Controller(controller.BaseController):
         for PARAM in SUPPORTED_PARAMS:
             if PARAM in req.params:
                 params[PARAM] = req.params.get(PARAM)
+
+        # Fix for LP Bug #1132294
+        # Ensure all shared images are returned in v1
+        params['member_status'] = 'all'
         return params
 
     def _get_filters(self, req):
@@ -274,7 +279,7 @@ class Controller(controller.BaseController):
                 if source.lower().startswith(scheme):
                     return source
             msg = _("External sourcing not supported for store %s") % source
-            LOG.error(msg)
+            LOG.debug(msg)
             raise HTTPBadRequest(explanation=msg,
                                  request=req,
                                  content_type="text/plain")
@@ -360,24 +365,25 @@ class Controller(controller.BaseController):
 
         try:
             image_meta = registry.add_image_metadata(req.context, image_meta)
+            self.notifier.info("image.create", redact_loc(image_meta))
             return image_meta
         except exception.Duplicate:
-            msg = (_("An image with identifier %s already exists")
-                  % image_meta['id'])
-            LOG.error(msg)
+            msg = (_("An image with identifier %s already exists") %
+                   image_meta['id'])
+            LOG.debug(msg)
             raise HTTPConflict(explanation=msg,
                                request=req,
                                content_type="text/plain")
         except exception.Invalid, e:
             msg = (_("Failed to reserve image. Got error: %(e)s") % locals())
             for line in msg.split('\n'):
-                LOG.error(line)
+                LOG.debug(line)
             raise HTTPBadRequest(explanation=msg,
                                  request=req,
                                  content_type="text/plain")
         except exception.Forbidden:
             msg = _("Forbidden to reserve image.")
-            LOG.error(msg)
+            LOG.debug(msg)
             raise HTTPForbidden(explanation=msg,
                                 request=req,
                                 content_type="text/plain")
@@ -404,7 +410,7 @@ class Controller(controller.BaseController):
             except Exception as e:
                 self._safe_kill(req, image_meta['id'])
                 msg = _("Copy from external source failed: %s") % e
-                LOG.error(msg)
+                LOG.debug(msg)
                 return
             image_meta['size'] = image_size or image_meta['size']
         else:
@@ -413,7 +419,7 @@ class Controller(controller.BaseController):
             except exception.InvalidContentType:
                 self._safe_kill(req, image_meta['id'])
                 msg = _("Content-Type must be application/octet-stream")
-                LOG.error(msg)
+                LOG.debug(msg)
                 raise HTTPBadRequest(explanation=msg)
 
             image_data = req.body_file
@@ -431,6 +437,7 @@ class Controller(controller.BaseController):
                     "to %(scheme)s store"), locals())
 
         try:
+            self.notifier.info("image.prepare", redact_loc(image_meta))
             location, size, checksum = store.add(
                 image_meta['id'],
                 utils.CooperativeReader(image_data),
@@ -465,22 +472,20 @@ class Controller(controller.BaseController):
             image_meta = registry.update_image_metadata(req.context,
                                                         image_id,
                                                         update_data)
-            self.notifier.info('image.upload', image_meta)
+            self.notifier.info('image.upload', redact_loc(image_meta))
 
             return location
 
         except exception.Duplicate, e:
             msg = _("Attempt to upload duplicate image: %s") % e
-            LOG.error(msg)
+            LOG.debug(msg)
             self._safe_kill(req, image_id)
-            self.notifier.error('image.upload', msg)
             raise HTTPConflict(explanation=msg, request=req)
 
         except exception.Forbidden, e:
             msg = _("Forbidden upload attempt: %s") % e
-            LOG.error(msg)
+            LOG.debug(msg)
             self._safe_kill(req, image_id)
-            self.notifier.error('image.upload', msg)
             raise HTTPForbidden(explanation=msg,
                                 request=req,
                                 content_type="text/plain")
@@ -502,14 +507,15 @@ class Controller(controller.BaseController):
                                          content_type='text/plain')
 
         except exception.ImageSizeLimitExceeded, e:
-            msg = _("Denying attempt to upload image larger than %d bytes.")
+            msg = _("Denying attempt to upload image larger than %d bytes."
+                    % CONF.image_size_cap)
+            LOG.info(msg)
             self._safe_kill(req, image_id)
-            raise HTTPBadRequest(explanation=msg % CONF.image_size_cap,
-                                 request=req, content_type='text/plain')
+            raise HTTPBadRequest(explanation=msg, request=req,
+                                 content_type='text/plain')
 
         except HTTPError, e:
             self._safe_kill(req, image_id)
-            self.notifier.error('image.upload', e.explanation)
             #NOTE(bcwaldon): Ideally, we would just call 'raise' here,
             # but something in the above function calls is affecting the
             # exception context and we must explicitly re-raise the
@@ -517,17 +523,9 @@ class Controller(controller.BaseController):
             raise e
 
         except Exception, e:
-            tb_info = traceback.format_exc()
-            LOG.error(tb_info)
-
+            LOG.exception(_("Failed to upload image"))
             self._safe_kill(req, image_id)
-
-            msg = _("Error uploading image: (%(class_name)s): "
-                    "%(exc)s") % ({'class_name': e.__class__.__name__,
-                    'exc': str(e)})
-
-            self.notifier.error('image.upload', msg)
-            raise HTTPBadRequest(explanation=msg, request=req)
+            raise HTTPInternalServerError(request=req)
 
     def _activate(self, req, image_id, location):
         """
@@ -544,16 +542,15 @@ class Controller(controller.BaseController):
 
         try:
             image_meta_data = registry.update_image_metadata(req.context,
-                                                  image_id,
-                                                  image_meta)
-            self.notifier.info("image.update", image_meta_data)
+                                                             image_id,
+                                                             image_meta)
+            self.notifier.info("image.activate", redact_loc(image_meta_data))
+            self.notifier.info("image.update", redact_loc(image_meta_data))
             return image_meta_data
         except exception.Invalid, e:
             msg = (_("Failed to activate image. Got error: %(e)s")
                    % locals())
-            for line in msg.split('\n'):
-                LOG.error(line)
-            self.notifier.error('image.update', msg)
+            LOG.debug(msg)
             raise HTTPBadRequest(explanation=msg,
                                  request=req,
                                  content_type="text/plain")
@@ -583,7 +580,7 @@ class Controller(controller.BaseController):
         except Exception, e:
             LOG.error(_("Unable to kill image %(id)s: "
                         "%(exc)s") % ({'id': image_id,
-                        'exc': repr(e)}))
+                                       'exc': repr(e)}))
 
     def _upload_and_activate(self, req, image_meta):
         """
@@ -611,8 +608,9 @@ class Controller(controller.BaseController):
 
     def _handle_source(self, req, image_id, image_meta, image_data):
         if image_data:
-            image_meta = self._validate_image_for_activation(req, image_id,
-                                                        image_meta)
+            image_meta = self._validate_image_for_activation(req,
+                                                             image_id,
+                                                             image_meta)
             image_meta = self._upload_and_activate(req, image_meta)
         elif self._copy_from(req):
             msg = _('Triggering asynchronous copy from external source')
@@ -628,11 +626,11 @@ class Controller(controller.BaseController):
     def _validate_image_for_activation(self, req, id, values):
         """Ensures that all required image metadata values are valid."""
         image = self.get_image_meta_or_404(req, id)
-        if not 'disk_format' in values:
+        if 'disk_format' not in values:
             values['disk_format'] = image['disk_format']
-        if not 'container_format' in values:
+        if 'container_format' not in values:
             values['container_format'] = image['container_format']
-        if not 'name' in values:
+        if 'name' not in values:
             values['name'] = image['name']
 
         values = validate_image_meta(req, values)
@@ -780,30 +778,26 @@ class Controller(controller.BaseController):
         except exception.Invalid, e:
             msg = (_("Failed to update image metadata. Got error: %(e)s")
                    % locals())
-            for line in msg.split('\n'):
-                LOG.error(line)
-            self.notifier.error('image.update', msg)
+            LOG.debug(msg)
             raise HTTPBadRequest(explanation=msg,
                                  request=req,
                                  content_type="text/plain")
         except exception.NotFound, e:
-            msg = ("Failed to find image to update: %(e)s" % locals())
+            msg = (_("Failed to find image to update: %(e)s") % locals())
             for line in msg.split('\n'):
                 LOG.info(line)
-            self.notifier.info('image.update', msg)
             raise HTTPNotFound(explanation=msg,
                                request=req,
                                content_type="text/plain")
         except exception.Forbidden, e:
-            msg = ("Forbidden to update image: %(e)s" % locals())
+            msg = (_("Forbidden to update image: %(e)s") % locals())
             for line in msg.split('\n'):
                 LOG.info(line)
-            self.notifier.info('image.update', msg)
             raise HTTPForbidden(explanation=msg,
                                 request=req,
                                 content_type="text/plain")
         else:
-            self.notifier.info('image.update', image_meta)
+            self.notifier.info('image.update', redact_loc(image_meta))
 
         # Prevent client from learning the location, as it
         # could contain security credentials
@@ -866,23 +860,22 @@ class Controller(controller.BaseController):
             if image['location']:
                 self._initiate_deletion(req, image['location'], id)
         except exception.NotFound, e:
-            msg = ("Failed to find image to delete: %(e)s" % locals())
+            msg = (_("Failed to find image to delete: %(e)s") % locals())
             for line in msg.split('\n'):
                 LOG.info(line)
-            self.notifier.info('image.delete', msg)
             raise HTTPNotFound(explanation=msg,
                                request=req,
                                content_type="text/plain")
         except exception.Forbidden, e:
-            msg = ("Forbidden to delete image: %(e)s" % locals())
+            msg = (_("Forbidden to delete image: %(e)s") % locals())
             for line in msg.split('\n'):
                 LOG.info(line)
-            self.notifier.info('image.delete', msg)
             raise HTTPForbidden(explanation=msg,
                                 request=req,
                                 content_type="text/plain")
         else:
-            self.notifier.info('image.delete', image)
+            self.notifier.info('image.delete', redact_loc(image))
+            return Response(body='', status=200)
 
     def get_store_or_400(self, request, scheme):
         """
@@ -897,29 +890,12 @@ class Controller(controller.BaseController):
         try:
             return get_store_from_scheme(request.context, scheme)
         except exception.UnknownScheme:
-            msg = _("Store for scheme %s not found")
-            LOG.error(msg % scheme)
+            msg = _("Store for scheme %s not found") % scheme
+            LOG.debug(msg)
             raise HTTPBadRequest(explanation=msg,
                                  request=request,
                                  content_type='text/plain')
 
-    def verify_scheme_or_exit(self, scheme):
-        """
-        Verifies availability of the storage backend for the
-        given scheme or exits
-
-        :param scheme: The backend store scheme
-        """
-        try:
-            get_store_from_scheme(context.RequestContext(), scheme)
-        except exception.UnknownScheme:
-            msg = _("Store for scheme %s not found")
-            LOG.error(msg % scheme)
-            # message on stderr will only be visible if started directly via
-            # bin/glance-api, as opposed to being daemonized by glance-control
-            sys.stderr.write(msg % scheme)
-            sys.exit(255)
-
 
 class ImageDeserializer(wsgi.JSONRequestDeserializer):
     """Handles deserialization of specific controller method requests."""
@@ -1013,7 +989,6 @@ class ImageSerializer(wsgi.JSONResponseSerializer):
 
     def show(self, response, result):
         image_meta = result['image_meta']
-        image_id = image_meta['id']
 
         image_iter = result['image_iterator']
         # image_meta['size'] should be an int, but could possibly be a str
