@@ -1,6 +1,6 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-# Copyright 2012 OpenStack LLC.
+# Copyright 2012 OpenStack Foundation.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -18,10 +18,10 @@
 import httplib
 import json
 
+from oslo.config import cfg
 import webob.dec
 
 from glance.common import wsgi
-from glance.openstack.common import cfg
 
 
 CONF = cfg.CONF
@@ -47,13 +47,15 @@ class Controller(object):
 
         version_objs = []
         if CONF.enable_v2_api:
-            version_objs.append(
-                build_version_object(2.0, 'v2', 'CURRENT'))
+            version_objs.extend([
+                build_version_object(2.1, 'v2', 'CURRENT'),
+                build_version_object(2.0, 'v2', 'SUPPORTED'),
+            ])
         if CONF.enable_v1_api:
-            version_objs.append(
-                build_version_object(1.1, 'v1', 'CURRENT'))
-            version_objs.append(
-                build_version_object(1.0, 'v1', 'SUPPORTED'))
+            version_objs.extend([
+                build_version_object(1.1, 'v1', 'CURRENT'),
+                build_version_object(1.0, 'v1', 'SUPPORTED'),
+            ])
 
         response = webob.Response(request=req,
                                   status=httplib.MULTIPLE_CHOICES,
