@@ -25,7 +25,6 @@ import functools
 import httplib
 import os
 import re
-import select
 import urllib
 import urlparse
 
@@ -44,6 +43,7 @@ except ImportError:
 from glance.common import auth
 from glance.common import exception, utils
 import glance.openstack.common.log as logging
+from glance.openstack.common import strutils
 
 LOG = logging.getLogger(__name__)
 
@@ -86,76 +86,6 @@ def handle_redirects(func):
     return wrapped
 
 
-class ImageBodyIterator(object):
-
-    """
-    A class that acts as an iterator over an image file's
-    chunks of data.  This is returned as part of the result
-    tuple from `glance.client.Client.get_image`
-    """
-
-    def __init__(self, source):
-        """
-        Constructs the object from a readable image source
-        (such as an HTTPResponse or file-like object)
-        """
-        self.source = source
-
-    def __iter__(self):
-        """
-        Exposes an iterator over the chunks of data in the
-        image file.
-        """
-        while True:
-            chunk = self.source.read(CHUNKSIZE)
-            if chunk:
-                yield chunk
-            else:
-                break
-
-
-class SendFileIterator:
-    """
-    Emulate iterator pattern over sendfile, in order to allow
-    send progress be followed by wrapping the iteration.
-    """
-    def __init__(self, connection, body):
-        self.connection = connection
-        self.body = body
-        self.offset = 0
-        self.sending = True
-
-    def __iter__(self):
-        class OfLength:
-            def __init__(self, len):
-                self.len = len
-
-            def __len__(self):
-                return self.len
-
-        while self.sending:
-            try:
-                sent = sendfile.sendfile(self.connection.sock.fileno(),
-                                         self.body.fileno(),
-                                         self.offset,
-                                         CHUNKSIZE)
-            except OSError as e:
-                # suprisingly, sendfile may fail transiently instead of
-                # blocking, in which case we select on the socket in order
-                # to wait on its return to a writeable state before resuming
-                # the send loop
-                if e.errno in (errno.EAGAIN, errno.EBUSY):
-                    wlist = [self.connection.sock.fileno()]
-                    rfds, wfds, efds = select.select([], wlist, [])
-                    if wfds:
-                        continue
-                raise
-
-            self.sending = (sent != 0)
-            self.offset += sent
-            yield OfLength(sent)
-
-
 class HTTPSClientAuthConnection(httplib.HTTPSConnection):
     """
     Class to make a HTTPS connection, with support for
@@ -268,6 +198,9 @@ class BaseClient(object):
         self.host = host
         self.port = port or self.DEFAULT_PORT
         self.timeout = timeout
+        # A value of '0' implies never timeout
+        if timeout == 0:
+            self.timeout = None
         self.use_ssl = use_ssl
         self.auth_tok = auth_tok
         self.creds = creds or {}
@@ -447,6 +380,7 @@ class BaseClient(object):
         """
         Create a URL object we can use to pass to _do_request().
         """
+        action = urllib.quote(action)
         path = '/'.join([self.doc_root or '', action.lstrip('/')])
         scheme = "https" if self.use_ssl else "http"
         netloc = "%s:%d" % (self.host, self.port)
@@ -455,6 +389,10 @@ class BaseClient(object):
             for (key, value) in params.items():
                 if value is None:
                     del params[key]
+                    continue
+                if not isinstance(value, basestring):
+                    value = str(value)
+                params[key] = strutils.safe_encode(value)
             query = urllib.urlencode(params)
         else:
             query = None
@@ -464,6 +402,20 @@ class BaseClient(object):
         LOG.debug(log_msg, url.geturl())
         return url
 
+    def _encode_headers(self, headers):
+        """
+        Encodes headers.
+
+        Note: This should be used right before
+        sending anything out.
+
+        :param headers: Headers to encode
+        :returns: Dictionary with encoded headers'
+                  names and values
+        """
+        to_str = strutils.safe_encode
+        return dict([(to_str(h), to_str(v)) for h, v in headers.iteritems()])
+
     @handle_redirects
     def _do_request(self, method, url, body, headers):
         """
@@ -494,7 +446,7 @@ class BaseClient(object):
 
         try:
             connection_type = self.get_connection_type()
-            headers = headers or {}
+            headers = self._encode_headers(headers or {})
 
             if 'x-auth-token' not in headers and self.auth_tok:
                 headers['x-auth-token'] = self.auth_tok
