@@ -15,8 +15,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import glance.common.utils
 from glance.openstack.common import local
+from glance.openstack.common import uuidutils
 
 
 class RequestContext(object):
@@ -36,7 +36,7 @@ class RequestContext(object):
         self.read_only = read_only
         self._show_deleted = show_deleted
         self.owner_is_tenant = owner_is_tenant
-        self.request_id = glance.common.utils.generate_uuid()
+        self.request_id = uuidutils.generate_uuid()
         self.service_catalog = service_catalog
 
         if not hasattr(local.store, 'context'):
@@ -45,15 +45,23 @@ class RequestContext(object):
     def to_dict(self):
         # NOTE(ameade): These keys are named to correspond with the default
         # format string for logging the context in openstack common
-        return {'request_id': self.request_id,
-                'user_id': self.user,
-                'tenant_id': self.tenant,
-                'is_admin': self.is_admin,
-                'project_id': self.tenant,
-                'read_deleted': self.show_deleted,
-                'roles': self.roles,
-                'auth_token': self.auth_tok,
-                'service_catalog': self.service_catalog,
+        return {
+            'request_id': self.request_id,
+
+            #NOTE(bcwaldon): openstack-common logging expects 'user'
+            'user': self.user,
+            'user_id': self.user,
+
+            #NOTE(bcwaldon): openstack-common logging expects 'tenant'
+            'tenant': self.tenant,
+            'tenant_id': self.tenant,
+            'project_id': self.tenant,
+
+            'is_admin': self.is_admin,
+            'read_deleted': self.show_deleted,
+            'roles': self.roles,
+            'auth_token': self.auth_tok,
+            'service_catalog': self.service_catalog,
         }
 
     @classmethod
