@@ -1,4 +1,4 @@
-# Copyright 2012 OpenStack, LLC
+# Copyright 2012 OpenStack, Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -13,13 +13,13 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import datetime
+import copy
 import functools
-import uuid
 
 from glance.common import exception
 import glance.openstack.common.log as logging
 from glance.openstack.common import timeutils
+from glance.openstack.common import uuidutils
 
 
 LOG = logging.getLogger(__name__)
@@ -34,10 +34,14 @@ DATA = {
 def log_call(func):
     @functools.wraps(func)
     def wrapped(*args, **kwargs):
-        LOG.info('Calling %s: args=%s, kwargs=%s' %
-                 (func.__name__, args, kwargs))
+        LOG.info(_('Calling %(funcname)s: args=%(args)s, kwargs=%(kwargs)s') %
+                 {"funcname": func.__name__,
+                  "args": args,
+                  "kwargs": kwargs})
         output = func(*args, **kwargs)
-        LOG.info('Returning %s: %s' % (func.__name__, output))
+        LOG.info(_('Returning %(funcname)s: %(output)s') %
+                 {"funcname": func.__name__,
+                  "output": output})
         return output
     return wrapped
 
@@ -51,6 +55,10 @@ def reset():
     }
 
 
+def setup_db_env(*args, **kwargs):
+    pass
+
+
 def configure_db(*args, **kwargs):
     pass
 
@@ -69,23 +77,26 @@ def _image_property_format(image_id, name, value):
     }
 
 
-def _image_member_format(image_id, tenant_id, can_share):
+def _image_member_format(image_id, tenant_id, can_share, status='pending'):
+    dt = timeutils.utcnow()
     return {
+        'id': uuidutils.generate_uuid(),
         'image_id': image_id,
         'member': tenant_id,
         'can_share': can_share,
-        'deleted': False,
-        'deleted_at': None,
+        'status': status,
+        'created_at': dt,
+        'updated_at': dt,
     }
 
 
 def _image_format(image_id, **values):
-    dt = datetime.datetime.now()
+    dt = timeutils.utcnow()
     image = {
         'id': image_id,
         'name': None,
         'owner': None,
-        'location': None,
+        'locations': [],
         'status': 'queued',
         'protected': False,
         'is_public': False,
@@ -113,15 +124,42 @@ def _image_format(image_id, **values):
     return image
 
 
-def _filter_images(images, filters, context):
+def _filter_images(images, filters, context, status='accepted'):
     filtered_images = []
     if 'properties' in filters:
         prop_filter = filters.pop('properties')
         filters.update(prop_filter)
 
-    is_public_filter = filters.pop('is_public', None)
+    if 'is_public' in filters and filters['is_public'] is None:
+        filters.pop('is_public')
 
-    for i, image in enumerate(images):
+    if status == 'all':
+        status = None
+
+    visibility = filters.pop('visibility', None)
+
+    for image in images:
+        member = image_member_find(context, image_id=image['id'],
+                                   member=context.owner, status=status)
+        is_member = len(member) > 0
+        has_ownership = context.owner and image['owner'] == context.owner
+        can_see = (image['is_public'] or has_ownership or context.is_admin or
+                   is_member)
+        if not can_see:
+            continue
+
+        if visibility:
+            if visibility == 'public':
+                filters['is_public'] = True
+                if not image['is_public']:
+                    continue
+            elif visibility == 'private':
+                filters['is_public'] = False
+                if not (has_ownership or context.is_admin):
+                    continue
+            elif visibility == 'shared':
+                if not is_member:
+                    continue
         add = True
         for k, value in filters.iteritems():
             key = k
@@ -144,41 +182,27 @@ def _filter_images(images, filters, context):
                 for p in image['properties']:
                     properties = {p['name']: p['value'],
                                   'deleted': p['deleted']}
-                add = properties.get(key) == value and \
-                                             properties.get('deleted') is False
+                add = (properties.get(key) == value and
+                       properties.get('deleted') is False)
             if not add:
                 break
 
         if add:
             filtered_images.append(image)
 
-    #NOTE(bcwaldon): This hacky code is only here to match what the
-    # sqlalchemy driver wants - refactor it out of the db layer when
-    # it seems appropriate
-    if is_public_filter is not None:
-        def _filter_ownership(image):
-            has_ownership = image['owner'] == context.owner
-            can_see = image['is_public'] or has_ownership or context.is_admin
-
-            if can_see:
-                return has_ownership or \
-                        (can_see and image['is_public'] == is_public_filter)
-            else:
-                return False
-
-        filtered_images = filter(_filter_ownership, filtered_images)
-
     return filtered_images
 
 
-def _do_pagination(context, images, marker, limit, show_deleted):
+def _do_pagination(context, images, marker, limit, show_deleted,
+                   status='accepted'):
     start = 0
     end = -1
     if marker is None:
         start = 0
     else:
         # Check that the image is accessible
-        image_get(context, marker, force_show_deleted=show_deleted)
+        _image_get(context, marker, force_show_deleted=show_deleted,
+                   status=status)
 
         for i, image in enumerate(images):
             if image['id'] == marker:
@@ -202,34 +226,37 @@ def _sort_images(images, sort_key, sort_dir):
     return images
 
 
-@log_call
-def image_get(context, image_id, session=None, force_show_deleted=False):
+def _image_get(context, image_id, force_show_deleted=False, status=None):
     try:
         image = DATA['images'][image_id]
     except KeyError:
-        LOG.info('Could not find image %s' % image_id)
+        LOG.info(_('Could not find image %s') % image_id)
         raise exception.NotFound()
 
     if image['deleted'] and not (force_show_deleted or context.show_deleted):
-        LOG.info('Unable to get deleted image')
+        LOG.info(_('Unable to get deleted image'))
         raise exception.NotFound()
 
-    if (not context.is_admin) \
-            and (not image.get('is_public')) \
-            and (image['owner'] is not None) \
-            and (image['owner'] != context.owner):
-        LOG.info('Unable to get unowned image')
-        raise exception.Forbidden()
+    if not is_image_visible(context, image):
+        LOG.info(_('Unable to get unowned image'))
+        raise exception.Forbidden("Image not visible to you")
 
     return image
 
 
+@log_call
+def image_get(context, image_id, session=None, force_show_deleted=False):
+    image = _image_get(context, image_id, force_show_deleted)
+    return copy.deepcopy(image)
+
+
 @log_call
 def image_get_all(context, filters=None, marker=None, limit=None,
-                  sort_key='created_at', sort_dir='desc'):
+                  sort_key='created_at', sort_dir='desc',
+                  member_status='accepted'):
     filters = filters or {}
     images = DATA['images'].values()
-    images = _filter_images(images, filters, context)
+    images = _filter_images(images, filters, context, member_status)
     images = _sort_images(images, sort_key, sort_dir)
     images = _do_pagination(context, images, marker, limit,
                             filters.get('deleted'))
@@ -238,9 +265,10 @@ def image_get_all(context, filters=None, marker=None, limit=None,
 
 @log_call
 def image_property_create(context, values):
-    image = image_get(context, values['image_id'])
-    prop = _image_property_format(values['image_id'], values['name'],
-                values['value'])
+    image = _image_get(context, values['image_id'])
+    prop = _image_property_format(values['image_id'],
+                                  values['name'],
+                                  values['value'])
     image['properties'].append(prop)
     return prop
 
@@ -254,54 +282,74 @@ def image_property_delete(context, prop_ref, session=None):
             prop = p
     if not prop:
         raise exception.NotFound()
-    prop['deleted_at'] = datetime.datetime.utcnow()
+    prop['deleted_at'] = timeutils.utcnow()
     prop['deleted'] = True
     return prop
 
 
 @log_call
-def image_member_find(context, image_id=None, member=None):
+def image_member_find(context, image_id=None, member=None, status=None):
     filters = []
+    images = DATA['images']
+    members = DATA['members']
+
+    def is_visible(member):
+        return (member['member'] == context.owner or
+                images[member['image_id']]['owner'] == context.owner)
+
+    if not context.is_admin:
+        filters.append(is_visible)
+
     if image_id is not None:
         filters.append(lambda m: m['image_id'] == image_id)
     if member is not None:
         filters.append(lambda m: m['member'] == member)
+    if status is not None:
+        filters.append(lambda m: m['status'] == status)
 
-    members = DATA['members']
     for f in filters:
         members = filter(f, members)
-    return members
+    return [copy.deepcopy(member) for member in members]
 
 
 @log_call
 def image_member_create(context, values):
     member = _image_member_format(values['image_id'],
                                   values['member'],
-                                  values.get('can_share', False))
+                                  values.get('can_share', False),
+                                  values.get('status', 'pending'))
     global DATA
     DATA['members'].append(member)
-    return member
+    return copy.deepcopy(member)
 
 
 @log_call
-def image_member_delete(context, member):
+def image_member_update(context, member_id, values):
     global DATA
-    mem = None
-    for p in DATA['members']:
-        if (p['member'] == member['member'] and
-            p['image_id'] == member['image_id']):
-            mem = p
-    if not mem:
+    for member in DATA['members']:
+        if (member['id'] == member_id):
+            member.update(values)
+            member['updated_at'] = timeutils.utcnow()
+            return copy.deepcopy(member)
+    else:
+        raise exception.NotFound()
+
+
+@log_call
+def image_member_delete(context, member_id):
+    global DATA
+    for i, member in enumerate(DATA['members']):
+        if (member['id'] == member_id):
+            del DATA['members'][i]
+            break
+    else:
         raise exception.NotFound()
-    mem['deleted_at'] = datetime.datetime.utcnow()
-    mem['deleted'] = True
-    return mem
 
 
 @log_call
 def image_create(context, image_values):
     global DATA
-    image_id = image_values.get('id', str(uuid.uuid4()))
+    image_id = image_values.get('id', uuidutils.generate_uuid())
 
     if image_id in DATA['images']:
         raise exception.Duplicate()
@@ -310,7 +358,7 @@ def image_create(context, image_values):
         raise exception.Invalid('status is a required attribute')
 
     allowed_keys = set(['id', 'name', 'status', 'min_ram', 'min_disk', 'size',
-                        'checksum', 'location', 'owner', 'protected',
+                        'checksum', 'locations', 'owner', 'protected',
                         'is_public', 'container_format', 'disk_format',
                         'created_at', 'updated_at', 'deleted_at', 'deleted',
                         'properties', 'tags'])
@@ -342,9 +390,8 @@ def image_update(context, image_id, image_values, purge_props=False):
             prop['deleted'] = True
 
     # add in any completly new properties
-    image['properties'].extend([
-            {'name': k, 'value': v, 'deleted': False}
-             for k, v in new_properties.items()])
+    image['properties'].extend([{'name': k, 'value': v, 'deleted': False}
+                                for k, v in new_properties.items()])
 
     image['updated_at'] = timeutils.utcnow()
     image.update(image_values)
@@ -357,14 +404,15 @@ def image_destroy(context, image_id):
     global DATA
     try:
         DATA['images'][image_id]['deleted'] = True
-        DATA['images'][image_id]['deleted_at'] = datetime.datetime.utcnow()
+        DATA['images'][image_id]['deleted_at'] = timeutils.utcnow()
+        return copy.deepcopy(DATA['images'][image_id])
     except KeyError:
         raise exception.NotFound()
 
 
 @log_call
 def image_tag_get_all(context, image_id):
-    image_get(context, image_id)
+    _image_get(context, image_id)
     return DATA['tags'].get(image_id, [])
 
 
@@ -447,7 +495,7 @@ def is_image_sharable(context, image, **kwargs):
     return member['can_share']
 
 
-def is_image_visible(context, image):
+def is_image_visible(context, image, status=None):
     """Return True if the image is visible in this context."""
     # Is admin == image visible
     if context.is_admin:
@@ -467,11 +515,14 @@ def is_image_visible(context, image):
             return True
 
         # Figure out if this image is shared with that tenant
+        if status == 'all':
+            status = None
         members = image_member_find(context,
                                     image_id=image['id'],
-                                    member=context.owner)
-        if members:
-            return not members[0]['deleted']
+                                    member=context.owner,
+                                    status=status)
+        if len(members) > 0:
+            return True
 
     # Private image
     return False
