@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright 2012 OpenStack Foundation.
 # All Rights Reserved.
 #
@@ -13,13 +15,14 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from oslo_log import log as logging
-from oslo_utils import encodeutils
-import six.moves.urllib.parse as urlparse
+import types
+import urlparse
+
 import sqlalchemy
 
 from glance.common import exception
-from glance.i18n import _, _LE
+import glance.openstack.common.log as logging
+import glance.store.swift
 
 LOG = logging.getLogger(__name__)
 
@@ -51,28 +54,16 @@ def migrate_location_credentials(migrate_engine, to_quoted):
                                       'swift')).execute())
 
     for image in images:
-        try:
-            fixed_uri = legacy_parse_uri(image['location'], to_quoted)
-            images_table.update().where(
-                images_table.c.id == image['id']).values(
-                    location=fixed_uri).execute()
-        except exception.BadStoreUri as e:
-            reason = encodeutils.exception_to_unicode(e)
-            msg = _LE("Invalid store uri for image: %(image_id)s. "
-                      "Details: %(reason)s") % {'image_id': image.id,
-                                                'reason': reason}
-            LOG.exception(msg)
-            raise
+        fixed_uri = fix_uri_credentials(image['location'], to_quoted)
+        images_table.update()\
+                    .where(images_table.c.id == image['id'])\
+                    .values(location=fixed_uri).execute()
 
 
-def legacy_parse_uri(uri, to_quote):
+def fix_uri_credentials(uri, to_quoted):
     """
-    Parse URLs. This method fixes an issue where credentials specified
-    in the URL are interpreted differently in Python 2.6.1+ than prior
-    versions of Python. It also deals with the peculiarity that new-style
-    Swift URIs have where a username can contain a ':', like so:
-
-        swift://account:user:pass@authurl.com/container/obj
+    Fix the given uri's embedded credentials by round-tripping with
+    StoreLocation.
 
     If to_quoted is True, the uri is assumed to have credentials that
     have not been quoted, and the resulting uri will contain quoted
@@ -82,6 +73,33 @@ def legacy_parse_uri(uri, to_quote):
     have been quoted, and the resulting uri will contain credentials
     that have not been quoted.
     """
+    location = glance.store.swift.StoreLocation({})
+    if to_quoted:
+        # The legacy parse_uri doesn't unquote credentials
+        location.parse_uri = types.MethodType(legacy_parse_uri, location)
+    else:
+        # The legacy _get_credstring doesn't quote credentials
+        location._get_credstring = types.MethodType(legacy__get_credstring,
+                                                    location)
+    location.parse_uri(uri)
+    return location.get_uri()
+
+
+def legacy__get_credstring(self):
+    if self.user:
+        return '%s:%s@' % (self.user, self.key)
+    return ''
+
+
+def legacy_parse_uri(self, uri):
+    """
+    Parse URLs. This method fixes an issue where credentials specified
+    in the URL are interpreted differently in Python 2.6.1+ than prior
+    versions of Python. It also deals with the peculiarity that new-style
+    Swift URIs have where a username can contain a ':', like so:
+
+        swift://account:user:pass@authurl.com/container/obj
+    """
     # Make sure that URIs that contain multiple schemes, such as:
     # swift://user:pass@http://authurl.com/v1/container/obj
     # are immediately rejected.
@@ -93,13 +111,12 @@ def legacy_parse_uri(uri, to_quote):
                    "like so: "
                    "swift+http://user:pass@authurl.com/v1/container/obj")
 
+        LOG.error(_("Invalid store uri %(uri)s: %(reason)s") % locals())
         raise exception.BadStoreUri(message=reason)
 
     pieces = urlparse.urlparse(uri)
-    if pieces.scheme not in ('swift', 'swift+http', 'swift+https'):
-        raise exception.BadStoreUri(message="Unacceptable scheme: '%s'" %
-                                    pieces.scheme)
-    scheme = pieces.scheme
+    assert pieces.scheme in ('swift', 'swift+http', 'swift+https')
+    self.scheme = pieces.scheme
     netloc = pieces.netloc
     path = pieces.path.lstrip('/')
     if netloc != '':
@@ -123,58 +140,29 @@ def legacy_parse_uri(uri, to_quote):
         # User can be account:user, in which case cred_parts[0:2] will be
         # the account and user. Combine them into a single username of
         # account:user
-        if to_quote:
-            if len(cred_parts) == 1:
-                reason = (_("Badly formed credentials '%(creds)s' in Swift "
-                            "URI") % {'creds': creds})
-                raise exception.BadStoreUri(message=reason)
-            elif len(cred_parts) == 3:
-                user = ':'.join(cred_parts[0:2])
-            else:
-                user = cred_parts[0]
-            key = cred_parts[-1]
-            user = user
-            key = key
+        if len(cred_parts) == 1:
+            reason = (_("Badly formed credentials '%(creds)s' in Swift "
+                        "URI") % locals())
+            LOG.error(reason)
+            raise exception.BadStoreUri()
+        elif len(cred_parts) == 3:
+            user = ':'.join(cred_parts[0:2])
         else:
-            if len(cred_parts) != 2:
-                reason = (_("Badly formed credentials in Swift URI."))
-                raise exception.BadStoreUri(message=reason)
-            user, key = cred_parts
-            user = urlparse.unquote(user)
-            key = urlparse.unquote(key)
+            user = cred_parts[0]
+        key = cred_parts[-1]
+        self.user = user
+        self.key = key
     else:
-        user = None
-        key = None
+        self.user = None
     path_parts = path.split('/')
     try:
-        obj = path_parts.pop()
-        container = path_parts.pop()
+        self.obj = path_parts.pop()
+        self.container = path_parts.pop()
         if not netloc.startswith('http'):
             # push hostname back into the remaining to build full authurl
             path_parts.insert(0, netloc)
-            auth_or_store_url = '/'.join(path_parts)
+            self.auth_or_store_url = '/'.join(path_parts)
     except IndexError:
-        reason = _("Badly formed S3 URI: %(uri)s") % {'uri': uri}
-        raise exception.BadStoreUri(message=reason)
-
-    if auth_or_store_url.startswith('http://'):
-        auth_or_store_url = auth_or_store_url[len('http://'):]
-    elif auth_or_store_url.startswith('https://'):
-        auth_or_store_url = auth_or_store_url[len('https://'):]
-
-    credstring = ''
-    if user and key:
-        if to_quote:
-            quote_user = urlparse.quote(user)
-            quote_key = urlparse.quote(key)
-        else:
-            quote_user = user
-            quote_key = key
-        credstring = '%s:%s@' % (quote_user, quote_key)
-
-    auth_or_store_url = auth_or_store_url.strip('/')
-    container = container.strip('/')
-    obj = obj.strip('/')
-
-    return '%s://%s%s/%s/%s' % (scheme, credstring, auth_or_store_url,
-                                container, obj)
+        reason = _("Badly formed S3 URI: %s") % uri
+        LOG.error(message=reason)
+        raise exception.BadStoreUri()
