@@ -21,9 +21,10 @@ Base attribute driver class
 
 import os.path
 
+from oslo.config import cfg
+
 from glance.common import exception
 from glance.common import utils
-from glance.openstack.common import cfg
 import glance.openstack.common.log as logging
 
 LOG = logging.getLogger(__name__)
