@@ -1,4 +1,6 @@
-# Copyright 2011 OpenStack Foundation
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
+# Copyright 2011 OpenStack LLC.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -16,59 +18,69 @@
 """
 Cache driver that uses SQLite to store information about cached images
 """
+
+from __future__ import absolute_import
 from contextlib import contextmanager
 import os
-import sqlite3
 import stat
 import time
 
-from oslo_concurrency import lockutils
-from oslo_config import cfg
-from oslo_log import log as logging
-from oslo_utils import excutils
-from oslo_utils import fileutils
+from eventlet import sleep, timeout
+from oslo.config import cfg
+import sqlite3
 
 from glance.common import exception
-from glance.i18n import _, _LI, _LW
 from glance.image_cache.drivers import base
-from glance.image_cache.drivers import common
-
+import glance.openstack.common.log as logging
 
 LOG = logging.getLogger(__name__)
 
-DEPRECATION_REASON = """
-As centralized database will now be used for image cache management, the use
-of `sqlite` database and driver will be dropped from 'E' (2025.1)
-development cycle.
-"""
-
 sqlite_opts = [
     cfg.StrOpt('image_cache_sqlite_db', default='cache.db',
-               deprecated_for_removal=True,
-               deprecated_reason=DEPRECATION_REASON,
-               deprecated_since='Caracal (2024.1)',
-               help=_("""
-The relative path to sqlite file database that will be used for image cache
-management.
-
-This is a relative path to the sqlite file database that tracks the age and
-usage statistics of image cache. The path is relative to image cache base
-directory, specified by the configuration option ``image_cache_dir``.
-
-This is a lightweight database with just one table.
-
-Possible values:
-    * A valid relative path to sqlite file database
-
-Related options:
-    * ``image_cache_dir``
-
-""")),
+               help=_('The path to the sqlite file database that will be '
+                      'used for image cache management.')),
 ]
 
 CONF = cfg.CONF
 CONF.register_opts(sqlite_opts)
 
+DEFAULT_SQL_CALL_TIMEOUT = 2
+
+
+class SqliteConnection(sqlite3.Connection):
+
+    """
+    SQLite DB Connection handler that plays well with eventlet,
+    slightly modified from Swift's similar code.
+    """
+
+    def __init__(self, *args, **kwargs):
+        self.timeout_seconds = kwargs.get('timeout', DEFAULT_SQL_CALL_TIMEOUT)
+        kwargs['timeout'] = 0
+        sqlite3.Connection.__init__(self, *args, **kwargs)
+
+    def _timeout(self, call):
+        with timeout.Timeout(self.timeout_seconds):
+            while True:
+                try:
+                    return call()
+                except sqlite3.OperationalError, e:
+                    if 'locked' not in str(e):
+                        raise
+                sleep(0.05)
+
+    def execute(self, *args, **kwargs):
+        return self._timeout(lambda: sqlite3.Connection.execute(
+                                        self, *args, **kwargs))
+
+    def commit(self):
+        return self._timeout(lambda: sqlite3.Connection.commit(self))
+
+
+def dict_factory(cur, row):
+    return dict(
+        ((col[0], row[idx]) for idx, col in enumerate(cur.description)))
+
 
 class Driver(base.Driver):
 
@@ -84,7 +96,6 @@ class Driver(base.Driver):
         this method. If the store was not able to successfully configure
         itself, it should raise `exception.BadDriverConfiguration`
         """
-        LOG.warning(_(DEPRECATION_REASON))
         super(Driver, self).configure()
 
         # Create the SQLite database that will hold our cache attributes
@@ -93,32 +104,26 @@ class Driver(base.Driver):
     def initialize_db(self):
         db = CONF.image_cache_sqlite_db
         self.db_path = os.path.join(self.base_dir, db)
-        lockutils.set_defaults(self.base_dir)
-
-        @lockutils.synchronized('image_cache_db_init', external=True)
-        def create_db():
-            try:
-                conn = sqlite3.connect(self.db_path, check_same_thread=False,
-                                       factory=common.SqliteConnection)
-                conn.executescript("""
-                    CREATE TABLE IF NOT EXISTS cached_images (
-                        image_id TEXT PRIMARY KEY,
-                        last_accessed REAL DEFAULT 0.0,
-                        last_modified REAL DEFAULT 0.0,
-                        size INTEGER DEFAULT 0,
-                        hits INTEGER DEFAULT 0,
-                        checksum TEXT
-                    );
-                """)
-                conn.close()
-            except sqlite3.DatabaseError as e:
-                msg = _("Failed to initialize the image cache database. "
-                        "Got error: %s") % e
-                LOG.error(msg)
-                raise exception.BadDriverConfiguration(driver_name='sqlite',
-                                                       reason=msg)
-
-        create_db()
+        try:
+            conn = sqlite3.connect(self.db_path, check_same_thread=False,
+                                   factory=SqliteConnection)
+            conn.executescript("""
+                CREATE TABLE IF NOT EXISTS cached_images (
+                    image_id TEXT PRIMARY KEY,
+                    last_accessed REAL DEFAULT 0.0,
+                    last_modified REAL DEFAULT 0.0,
+                    size INTEGER DEFAULT 0,
+                    hits INTEGER DEFAULT 0,
+                    checksum TEXT
+                );
+            """)
+            conn.close()
+        except sqlite3.DatabaseError, e:
+            msg = _("Failed to initialize the image cache database. "
+                    "Got error: %s") % e
+            LOG.error(msg)
+            raise exception.BadDriverConfiguration(driver_name='sqlite',
+                                                   reason=msg)
 
     def get_cache_size(self):
         """
@@ -142,7 +147,7 @@ class Driver(base.Driver):
             return 0
 
         hits = 0
-        with common.get_db(self.db_path) as db:
+        with self.get_db() as db:
             cur = db.execute("""SELECT hits FROM cached_images
                              WHERE image_id = ?""",
                              (image_id,))
@@ -153,13 +158,13 @@ class Driver(base.Driver):
         """
         Returns a list of records about cached images.
         """
-        LOG.debug("Gathering cached image entries.")
-        with common.get_db(self.db_path) as db:
+        LOG.debug(_("Gathering cached image entries."))
+        with self.get_db() as db:
             cur = db.execute("""SELECT
                              image_id, hits, last_accessed, last_modified, size
                              FROM cached_images
                              ORDER BY image_id""")
-            cur.row_factory = common.dict_factory
+            cur.row_factory = dict_factory
             return [r for r in cur]
 
     def is_cached(self, image_id):
@@ -206,7 +211,7 @@ class Driver(base.Driver):
         Removes all cached image files and any attributes about the images
         """
         deleted = 0
-        with common.get_db(self.db_path) as db:
+        with self.get_db() as db:
             for path in self.get_cache_files(self.base_dir):
                 delete_cached_file(path)
                 deleted += 1
@@ -221,7 +226,7 @@ class Driver(base.Driver):
         :param image_id: Image ID
         """
         path = self.get_image_filepath(image_id)
-        with common.get_db(self.db_path) as db:
+        with self.get_db() as db:
             delete_cached_file(path)
             db.execute("""DELETE FROM cached_images WHERE image_id = ?""",
                        (image_id, ))
@@ -233,7 +238,7 @@ class Driver(base.Driver):
         """
         files = [f for f in self.get_cache_files(self.queue_dir)]
         for file in files:
-            fileutils.delete_if_exists(file)
+            os.unlink(file)
         return len(files)
 
     def delete_queued_image(self, image_id):
@@ -243,7 +248,8 @@ class Driver(base.Driver):
         :param image_id: Image ID
         """
         path = self.get_image_filepath(image_id, 'queue')
-        fileutils.delete_if_exists(path)
+        if os.path.exists(path):
+            os.unlink(path)
 
     def clean(self, stall_time=None):
         """
@@ -265,7 +271,7 @@ class Driver(base.Driver):
         Return a tuple containing the image_id and size of the least recently
         accessed cached file, or None if no cached files.
         """
-        with common.get_db(self.db_path) as db:
+        with self.get_db() as db:
             cur = db.execute("""SELECT image_id FROM cached_images
                              ORDER BY last_accessed LIMIT 1""")
             try:
@@ -275,12 +281,8 @@ class Driver(base.Driver):
                 return None
 
         path = self.get_image_filepath(image_id)
-        try:
-            file_info = os.stat(path)
-            size = file_info[stat.ST_SIZE]
-        except OSError:
-            size = 0
-        return image_id, size
+        file_info = os.stat(path)
+        return image_id, file_info[stat.ST_SIZE]
 
     @contextmanager
     def open_for_write(self, image_id):
@@ -293,40 +295,35 @@ class Driver(base.Driver):
         incomplete_path = self.get_image_filepath(image_id, 'incomplete')
 
         def commit():
-            with common.get_db(self.db_path) as db:
+            with self.get_db() as db:
                 final_path = self.get_image_filepath(image_id)
-                LOG.debug("Fetch finished, moving "
-                          "'%(incomplete_path)s' to '%(final_path)s'",
+                LOG.debug(_("Fetch finished, moving "
+                          "'%(incomplete_path)s' to '%(final_path)s'"),
                           dict(incomplete_path=incomplete_path,
                                final_path=final_path))
                 os.rename(incomplete_path, final_path)
 
                 # Make sure that we "pop" the image from the queue...
                 if self.is_queued(image_id):
-                    fileutils.delete_if_exists(
-                        self.get_image_filepath(image_id, 'queue'))
+                    os.unlink(self.get_image_filepath(image_id, 'queue'))
 
                 filesize = os.path.getsize(final_path)
                 now = time.time()
 
                 db.execute("""INSERT INTO cached_images
                            (image_id, last_accessed, last_modified, hits, size)
-                           VALUES (?, ?, ?, 0, ?)""",
-                           (image_id, now, now, filesize))
+                           VALUES (?, 0, ?, 0, ?)""",
+                           (image_id, now, filesize))
                 db.commit()
 
         def rollback(e):
-            with common.get_db(self.db_path) as db:
+            with self.get_db() as db:
                 if os.path.exists(incomplete_path):
                     invalid_path = self.get_image_filepath(image_id, 'invalid')
 
-                    msg = (_LW("Fetch of cache file failed (%(e)s), rolling "
-                               "back by moving '%(incomplete_path)s' to "
-                               "'%(invalid_path)s'"),
-                           {'e': e,
-                            'incomplete_path': incomplete_path,
-                            'invalid_path': invalid_path})
-                    LOG.warning(msg)
+                    LOG.debug(_("Fetch of cache file failed (%(e)s), rolling "
+                                "back by moving '%(incomplete_path)s' to "
+                                "'%(invalid_path)s'") % locals())
                     os.rename(incomplete_path, invalid_path)
 
                 db.execute("""DELETE FROM cached_images
@@ -337,8 +334,8 @@ class Driver(base.Driver):
             with open(incomplete_path, 'wb') as cache_file:
                 yield cache_file
         except Exception as e:
-            with excutils.save_and_reraise_exception():
-                rollback(e)
+            rollback(e)
+            raise
         else:
             commit()
         finally:
@@ -362,13 +359,36 @@ class Driver(base.Driver):
         with open(path, 'rb') as cache_file:
             yield cache_file
         now = time.time()
-        with common.get_db(self.db_path) as db:
+        with self.get_db() as db:
             db.execute("""UPDATE cached_images
                        SET hits = hits + 1, last_accessed = ?
                        WHERE image_id = ?""",
                        (now, image_id))
             db.commit()
 
+    @contextmanager
+    def get_db(self):
+        """
+        Returns a context manager that produces a database connection that
+        self-closes and calls rollback if an error occurs while using the
+        database connection
+        """
+        conn = sqlite3.connect(self.db_path, check_same_thread=False,
+                               factory=SqliteConnection)
+        conn.row_factory = sqlite3.Row
+        conn.text_factory = str
+        conn.execute('PRAGMA synchronous = NORMAL')
+        conn.execute('PRAGMA count_changes = OFF')
+        conn.execute('PRAGMA temp_store = MEMORY')
+        try:
+            yield conn
+        except sqlite3.DatabaseError, e:
+            msg = _("Error executing SQLite call. Got error: %s") % e
+            LOG.error(msg)
+            conn.rollback()
+        finally:
+            conn.close()
+
     def queue_image(self, image_id):
         """
         This adds a image to be cache to the queue.
@@ -379,22 +399,25 @@ class Driver(base.Driver):
         :param image_id: Image ID
         """
         if self.is_cached(image_id):
-            LOG.info(_LI("Not queueing image '%s'. Already cached."), image_id)
+            msg = _("Not queueing image '%s'. Already cached.") % image_id
+            LOG.warn(msg)
             return False
 
         if self.is_being_cached(image_id):
-            LOG.info(_LI("Not queueing image '%s'. Already being "
-                         "written to cache"), image_id)
+            msg = _("Not queueing image '%s'. Already being "
+                    "written to cache") % image_id
+            LOG.warn(msg)
             return False
 
         if self.is_queued(image_id):
-            LOG.info(_LI("Not queueing image '%s'. Already queued."), image_id)
+            msg = _("Not queueing image '%s'. Already queued.") % image_id
+            LOG.warn(msg)
             return False
 
         path = self.get_image_filepath(image_id, 'queue')
 
         # Touch the file to add it to the queue
-        with open(path, "w"):
+        with open(path, "w") as f:
             pass
 
         return True
@@ -404,27 +427,20 @@ class Driver(base.Driver):
         Removes any invalid cache entries
         """
         for path in self.get_cache_files(self.invalid_dir):
-            fileutils.delete_if_exists(path)
-            LOG.info(_LI("Removed invalid cache file %s"), path)
+            os.unlink(path)
+            LOG.info(_("Removed invalid cache file %s"), path)
 
     def delete_stalled_files(self, older_than):
         """
         Removes any incomplete cache entries older than a
         supplied modified time.
 
-        :param older_than: Files written to on or before this timestamp
+        :param older_than: Files written to on or before this timestemp
                            will be deleted.
         """
         for path in self.get_cache_files(self.incomplete_dir):
-            if os.path.getmtime(path) < older_than:
-                try:
-                    fileutils.delete_if_exists(path)
-                    LOG.info(_LI("Removed stalled cache file %s"), path)
-                except Exception as e:
-                    msg = (_LW("Failed to delete file %(path)s. "
-                               "Got error: %(e)s"),
-                           dict(path=path, e=e))
-                    LOG.warning(msg)
+            os.unlink(path)
+            LOG.info(_("Removed stalled cache file %s"), path)
 
     def get_queued_images(self):
         """
@@ -439,7 +455,7 @@ class Driver(base.Driver):
             items.append((mtime, os.path.basename(path)))
 
         items.sort()
-        return [image_id for (modtime, image_id) in items]
+        return [image_id for (mtime, image_id) in items]
 
     def get_cache_files(self, basepath):
         """
@@ -454,5 +470,9 @@ class Driver(base.Driver):
 
 
 def delete_cached_file(path):
-    LOG.debug("Deleting image cache file '%s'", path)
-    fileutils.delete_if_exists(path)
+    if os.path.exists(path):
+        LOG.debug(_("Deleting image cache file '%s'"), path)
+        os.unlink(path)
+    else:
+        LOG.warn(_("Cached image file '%s' doesn't exist, unable to"
+                   " delete"), path)
