@@ -16,10 +16,10 @@
 
 import json
 
+from oslo.config import cfg
 import qpid.messaging
 
 from glance.notifier import strategy
-from glance.openstack.common import cfg
 import glance.openstack.common.log as logging
 
 LOG = logging.getLogger(__name__)
@@ -29,7 +29,7 @@ qpid_opts = [
                default='glance',
                help='Qpid exchange for notifications'),
     cfg.StrOpt('qpid_notification_topic',
-               default='glance_notifications',
+               default='notifications',
                help='Qpid topic for notifications'),
     cfg.StrOpt('qpid_hostname',
                default='localhost',
@@ -42,7 +42,8 @@ qpid_opts = [
                help='Username for qpid connection'),
     cfg.StrOpt('qpid_password',
                default='',
-               help='Password for qpid connection'),
+               help='Password for qpid connection',
+               secret=True),
     cfg.StrOpt('qpid_sasl_mechanisms',
                default='',
                help='Space separated list of SASL mechanisms to use for auth'),
@@ -70,7 +71,7 @@ qpid_opts = [
     cfg.BoolOpt('qpid_tcp_nodelay',
                 default=True,
                 help='Disable Nagle algorithm'),
-    ]
+]
 
 CONF = cfg.CONF
 CONF.register_opts(qpid_opts)
@@ -79,40 +80,36 @@ CONF.register_opts(qpid_opts)
 class QpidStrategy(strategy.Strategy):
     """A notifier that puts a message on a queue when called."""
 
-    def __init__(self):
+    def _open_connection(self):
         """Initialize the Qpid notification strategy."""
-        self.broker = CONF.qpid_hostname + ":" + CONF.qpid_port
-        self.connection = qpid.messaging.Connection(self.broker)
-        self.connection.username = CONF.qpid_username
-        self.connection.password = CONF.qpid_password
-        self.connection.sasl_mechanisms = CONF.qpid_sasl_mechanisms
+        broker = CONF.qpid_hostname + ":" + CONF.qpid_port
+        connection = qpid.messaging.Connection(broker)
+        connection.username = CONF.qpid_username
+        connection.password = CONF.qpid_password
+        connection.sasl_mechanisms = CONF.qpid_sasl_mechanisms
         # Hard code this option as enabled so that reconnect logic isn't needed
         # in this file at all.
-        self.connection.reconnect = True
+        connection.reconnect = True
         if CONF.qpid_reconnect_timeout:
-            self.connection.reconnect_timeout = CONF.qpid_reconnect_timeout
+            connection.reconnect_timeout = CONF.qpid_reconnect_timeout
         if CONF.qpid_reconnect_limit:
-            self.connection.reconnect_limit = CONF.qpid_reconnect_limit
+            connection.reconnect_limit = CONF.qpid_reconnect_limit
         if CONF.qpid_reconnect_interval_max:
-            self.connection.reconnect_interval_max = (
+            connection.reconnect_interval_max = (
                                         CONF.qpid_reconnect_interval_max)
         if CONF.qpid_reconnect_interval_min:
-            self.connection.reconnect_interval_min = (
+            connection.reconnect_interval_min = (
                                         CONF.qpid_reconnect_interval_min)
         if CONF.qpid_reconnect_interval:
-            self.connection.reconnect_interval = CONF.qpid_reconnect_interval
-        self.connection.heartbeat = CONF.qpid_heartbeat
-        self.connection.protocol = CONF.qpid_protocol
-        self.connection.tcp_nodelay = CONF.qpid_tcp_nodelay
-        self.connection.open()
-        self.session = self.connection.session()
-        LOG.info(_('Connected to AMQP server on %s') % self.broker)
+            connection.reconnect_interval = CONF.qpid_reconnect_interval
+        connection.heartbeat = CONF.qpid_heartbeat
+        connection.protocol = CONF.qpid_protocol
+        connection.tcp_nodelay = CONF.qpid_tcp_nodelay
+        connection.open()
+        LOG.info(_('Connected to AMQP server on %s') % broker)
+        return connection
 
-        self.sender_info = self._sender("info")
-        self.sender_warn = self._sender("warn")
-        self.sender_error = self._sender("error")
-
-    def _sender(self, priority):
+    def _send(self, priority, msg):
         addr_opts = {
             "create": "always",
             "node": {
@@ -128,16 +125,27 @@ class QpidStrategy(strategy.Strategy):
         topic = "%s.%s" % (CONF.qpid_notification_topic, priority)
         address = "%s/%s ; %s" % (CONF.qpid_notification_exchange, topic,
                                   json.dumps(addr_opts))
-        return self.session.sender(address)
+        connection = None
+        try:
+            connection = self._open_connection()
+            session = connection.session()
+            sender = session.sender(address)
+            qpid_msg = qpid.messaging.Message(content=msg)
+            sender.send(qpid_msg)
+        except Exception:
+            details = dict(priority=priority, msg=msg)
+            LOG.exception(_('Notification error.  Priority: %(priority)s '
+                            'Message: %(msg)s' % details))
+            raise
+        finally:
+            if connection and connection.opened():
+                connection.close()
 
     def warn(self, msg):
-        qpid_msg = qpid.messaging.Message(content=msg)
-        self.sender_warn.send(qpid_msg)
+        self._send('warn', msg)
 
     def info(self, msg):
-        qpid_msg = qpid.messaging.Message(content=msg)
-        self.sender_info.send(qpid_msg)
+        self._send('info', msg)
 
     def error(self, msg):
-        qpid_msg = qpid.messaging.Message(content=msg)
-        self.sender_error.send(qpid_msg)
+        self._send('error', msg)
