@@ -21,10 +21,13 @@ the Glance Registry API
 """
 
 import json
-import urllib
 
 from glance.common.client import BaseClient
-from glance.registry import server
+from glance.common import crypt
+import glance.openstack.common.log as logging
+from glance.registry.api.v1 import images
+
+LOG = logging.getLogger(__name__)
 
 
 class RegistryClient(BaseClient):
@@ -33,6 +36,36 @@ class RegistryClient(BaseClient):
 
     DEFAULT_PORT = 9191
 
+    def __init__(self, host=None, port=None, metadata_encryption_key=None,
+                 **kwargs):
+        """
+        :param metadata_encryption_key: Key used to encrypt 'location' metadata
+        """
+        self.metadata_encryption_key = metadata_encryption_key
+        # NOTE (dprince): by default base client overwrites host and port
+        # settings when using keystone. configure_via_auth=False disables
+        # this behaviour to ensure we still send requests to the Registry API
+        BaseClient.__init__(self, host, port, configure_via_auth=False,
+                            **kwargs)
+
+    def decrypt_metadata(self, image_metadata):
+        if (self.metadata_encryption_key is not None and
+            'location' in image_metadata and
+            image_metadata['location'] is not None):
+            location = crypt.urlsafe_decrypt(self.metadata_encryption_key,
+                                             image_metadata['location'])
+            image_metadata['location'] = location
+        return image_metadata
+
+    def encrypt_metadata(self, image_metadata):
+        if (self.metadata_encryption_key is not None and
+            'location' in image_metadata and
+            image_metadata['location'] is not None):
+            location = crypt.urlsafe_encrypt(self.metadata_encryption_key,
+                                             image_metadata['location'], 64)
+            image_metadata['location'] = location
+        return image_metadata
+
     def get_images(self, **kwargs):
         """
         Returns a list of image id/name mappings from Registry
@@ -43,10 +76,30 @@ class RegistryClient(BaseClient):
         :param sort_key: results will be ordered by this image attribute
         :param sort_dir: direction in which to to order results (asc, desc)
         """
-        params = self._extract_params(kwargs, server.SUPPORTED_PARAMS)
+        params = self._extract_params(kwargs, images.SUPPORTED_PARAMS)
         res = self.do_request("GET", "/images", params=params)
-        data = json.loads(res.read())['images']
-        return data
+        image_list = json.loads(res.read())['images']
+        for image in image_list:
+            image = self.decrypt_metadata(image)
+        return image_list
+
+    def do_request(self, method, action, **kwargs):
+        try:
+            res = super(RegistryClient, self).do_request(method,
+                                                         action,
+                                                         **kwargs)
+            status = res.status
+            request_id = res.getheader('x-openstack-request-id')
+            msg = _("Registry request %(method)s %(action)s HTTP %(status)s"
+                    " request id %(request_id)s")
+            LOG.debug(msg % locals())
+
+        except Exception as exc:
+            exc_name = exc.__class__.__name__
+            LOG.info(_("Registry client request %(method)s %(action)s "
+                       "raised %(exc_name)s") % locals())
+            raise
+        return res
 
     def get_images_detailed(self, **kwargs):
         """
@@ -58,16 +111,18 @@ class RegistryClient(BaseClient):
         :param sort_key: results will be ordered by this image attribute
         :param sort_dir: direction in which to to order results (asc, desc)
         """
-        params = self._extract_params(kwargs, server.SUPPORTED_PARAMS)
+        params = self._extract_params(kwargs, images.SUPPORTED_PARAMS)
         res = self.do_request("GET", "/images/detail", params=params)
-        data = json.loads(res.read())['images']
-        return data
+        image_list = json.loads(res.read())['images']
+        for image in image_list:
+            image = self.decrypt_metadata(image)
+        return image_list
 
     def get_image(self, image_id):
         """Returns a mapping of image metadata from Registry"""
         res = self.do_request("GET", "/images/%s" % image_id)
         data = json.loads(res.read())['image']
-        return data
+        return self.decrypt_metadata(data)
 
     def add_image(self, image_metadata):
         """
@@ -77,23 +132,28 @@ class RegistryClient(BaseClient):
             'Content-Type': 'application/json',
         }
 
-        if 'image' not in image_metadata.keys():
+        if 'image' not in image_metadata:
             image_metadata = dict(image=image_metadata)
 
+        encrypted_metadata = self.encrypt_metadata(image_metadata['image'])
+        image_metadata['image'] = encrypted_metadata
         body = json.dumps(image_metadata)
 
-        res = self.do_request("POST", "/images", body, headers=headers)
+        res = self.do_request("POST", "/images", body=body, headers=headers)
         # Registry returns a JSONified dict(image=image_info)
         data = json.loads(res.read())
-        return data['image']
+        image = data['image']
+        return self.decrypt_metadata(image)
 
     def update_image(self, image_id, image_metadata, purge_props=False):
         """
         Updates Registry's information about an image
         """
-        if 'image' not in image_metadata.keys():
+        if 'image' not in image_metadata:
             image_metadata = dict(image=image_metadata)
 
+        encrypted_metadata = self.encrypt_metadata(image_metadata['image'])
+        image_metadata['image'] = encrypted_metadata
         body = json.dumps(image_metadata)
 
         headers = {
@@ -103,17 +163,20 @@ class RegistryClient(BaseClient):
         if purge_props:
             headers["X-Glance-Registry-Purge-Props"] = "true"
 
-        res = self.do_request("PUT", "/images/%s" % image_id, body, headers)
+        res = self.do_request("PUT", "/images/%s" % image_id, body=body,
+                              headers=headers)
         data = json.loads(res.read())
         image = data['image']
-        return image
+        return self.decrypt_metadata(image)
 
     def delete_image(self, image_id):
         """
         Deletes Registry's information about an image
         """
-        self.do_request("DELETE", "/images/%s" % image_id)
-        return True
+        res = self.do_request("DELETE", "/images/%s" % image_id)
+        data = json.loads(res.read())
+        image = data['image']
+        return image
 
     def get_image_members(self, image_id):
         """Returns a list of membership associations from Registry"""
@@ -140,8 +203,8 @@ class RegistryClient(BaseClient):
         headers = {'Content-Type': 'application/json', }
 
         res = self.do_request("PUT", "/images/%s/members" % image_id,
-                              body, headers)
-        return res.status == 204
+                              body=body, headers=headers)
+        return self.get_status_code(res) == 204
 
     def add_member(self, image_id, member_id, can_share=None):
         """Adds to Registry's information about image membership"""
@@ -152,12 +215,13 @@ class RegistryClient(BaseClient):
             body = json.dumps(dict(member=dict(can_share=can_share)))
             headers['Content-Type'] = 'application/json'
 
-        res = self.do_request("PUT", "/images/%s/members/%s" %
-                              (image_id, member_id), body, headers)
-        return res.status == 204
+        url = "/images/%s/members/%s" % (image_id, member_id)
+        res = self.do_request("PUT", url, body=body,
+                              headers=headers)
+        return self.get_status_code(res) == 204
 
     def delete_member(self, image_id, member_id):
         """Deletes Registry's information about image membership"""
         res = self.do_request("DELETE", "/images/%s/members/%s" %
                               (image_id, member_id))
-        return res.status == 204
+        return self.get_status_code(res) == 204
