@@ -84,11 +84,7 @@ def setup_http(test):
     test.http_server = remote_server
     test.http_ip = remote_ip
     test.http_port = remote_port
-
-
-def teardown_http(test):
-    if test.http_server:
-        test.http_server.shutdown()
+    test.addCleanup(test.http_server.shutdown)
 
 
 def get_http_uri(test, image_id):
@@ -200,7 +196,7 @@ def get_swift_uri(test, image_id):
     uri = ('swift+https://%(swift_store_user)s:%(swift_store_key)s' %
            test.__dict__)
     uri += ('@%(swift_store_auth_address)s/%(swift_store_container)s/' %
-           test.__dict__)
+            test.__dict__)
     uri += image_id
     return uri.replace('@http://', '@')
 
