@@ -90,8 +90,8 @@ class TestClientRedirects(functional.FunctionalTest):
         server_two = wsgi.Server()
         self.config(bind_host='127.0.0.1')
         self.config(workers=0)
-        server_one.start(RedirectTestApp("one"), self.port_one)
-        server_two.start(RedirectTestApp("two"), self.port_two)
+        server_one.start(RedirectTestApp("one")(), self.port_one)
+        server_two.start(RedirectTestApp("two")(), self.port_two)
         self.client = client.BaseClient("127.0.0.1", self.port_one)
 
     def test_get_without_redirect(self):
@@ -114,7 +114,8 @@ class TestClientRedirects(functional.FunctionalTest):
         """
         Test GET with one 302 FOUND redirect w/ a query string
         """
-        response = self.client.do_request("GET", "/302?with_qs=yes")
+        response = self.client.do_request("GET", "/302",
+                                          params={'with_qs': 'yes'})
         self.assertEquals(200, response.status)
         self.assertEquals("success_with_qs", response.read())
 
