@@ -1,6 +1,6 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-# Copyright 2012 OpenStack, LLC
+# Copyright 2012 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -43,10 +43,12 @@ class TestMultiprocessing(functional.FunctionalTest):
         self.stop_servers()
 
     def _get_children(self):
-        cmd = ("ps -fu $USER | grep glance-api | "
-               "grep -v grep | awk '{print $2}' | sort -nr")
-        _, out, _ = execute(cmd, raise_error=True)
-        return out.split('\n')[0:2]
+        api_pid = open(self.api_server.pid_file).read().strip()
+        cmd = ("ps --no-headers --ppid %s -o pid,cmd | "
+               "grep python | "  # NOTE(markwash): ignore non-python procs
+               "awk '{print $1; print >> \"/dev/stderr\"}'" % api_pid)
+        _, out, err = execute(cmd, raise_error=True)
+        return out.strip().split('\n')
 
     def test_interrupt_avoids_respawn_storm(self):
         """
@@ -68,6 +70,6 @@ class TestMultiprocessing(functional.FunctionalTest):
             self.assertTrue(len(children) >= len(self._get_children()))
             for child in self._get_children():
                 # ensure no new children spawned
-                self.assertTrue(child in children)
+                self.assertTrue(child in children, child)
 
         self.stop_servers()
