@@ -7,8 +7,10 @@ function usage {
   echo "  -V, --virtual-env        Always use virtualenv.  Install automatically if not present"
   echo "  -N, --no-virtual-env     Don't use virtualenv.  Run tests in local environment"
   echo "  -f, --force              Force a clean re-build of the virtual environment. Useful when dependencies have been added."
+  echo "  -u, --update             Update the virtual environment with any newer package versions"
   echo "  --unittests-only         Run unit tests only, exclude functional tests."
   echo "  -p, --pep8               Just run pep8"
+  echo "  -P, --no-pep8            Don't run static code checks"
   echo "  -h, --help               Print this usage message"
   echo ""
   echo "Note: with no options specified, the script will try to run the tests in a virtual environment,"
@@ -23,7 +25,9 @@ function process_option {
     -V|--virtual-env) let always_venv=1; let never_venv=0;;
     -N|--no-virtual-env) let always_venv=0; let never_venv=1;;
     -p|--pep8) let just_pep8=1;;
+    -P|--no-pep8) let no_pep8=1;;
     -f|--force) let force=1;;
+    -u|--update) update=1;;
     --unittests-only) noseopts="$noseopts --exclude-dir=glance/tests/functional";;
     -c|--coverage) noseopts="$noseopts --with-coverage --cover-package=glance";;
     -*) noseopts="$noseopts $1";;
@@ -40,6 +44,8 @@ noseopts=
 noseargs=
 wrapper=""
 just_pep8=0
+no_pep8=0
+update=0
 
 export NOSE_WITH_OPENSTACK=1
 export NOSE_OPENSTACK_COLOR=1
@@ -64,8 +70,10 @@ function run_pep8 {
   echo "Running pep8 ..."
   PEP8_EXCLUDE=".venv,.tox,dist,doc,openstack"
   PEP8_OPTIONS="--exclude=$PEP8_EXCLUDE --repeat"
-  PEP8_INCLUDE="."
-  ${wrapper} pep8 $PEP8_OPTIONS $PEP8_INCLUDE
+  PEP8_IGNORE="--ignore=E125,E126,E711,E712"
+  PEP8_INCLUDE=". bin/*"
+
+  ${wrapper} pep8 $PEP8_OPTIONS $PEP8_INCLUDE $PEP8_IGNORE
 }
 
 
@@ -78,6 +86,10 @@ then
     echo "Cleaning virtualenv..."
     rm -rf ${venv}
   fi
+  if [ $update -eq 1 ]; then
+    echo "Updating virtualenv..."
+    python tools/install_venv.py
+  fi
   if [ -e ${venv} ]; then
     wrapper="${with_venv}"
   else
@@ -105,5 +117,7 @@ fi
 run_tests || exit
 
 if [ -z "$noseargs" ]; then
-  run_pep8
+    if [ $no_pep8 -eq 0 ]; then
+        run_pep8
+    fi
 fi
